/**
 * @file
 * A Backbone view for the edit_ui block element.
 */

(function (Drupal, Backbone, $, Modernizr) {
  "use strict";

  var strings = {
    confirmDelete: Drupal.t('Are you sure you want to delete the block "@name"?')
  };

  /**
   * Backbone view for the edit_ui block.
   */
  Drupal.editUiBlock.BlockVisualView = Backbone.View.extend({
    /**
     * Custom data.
     */
    hiddenClass: 'visually-hidden',

    /**
     * Dom elements events.
     */
    events: function() {
      var events = {
        "click .edit-uiblockdelete": "deleteBlock"
      };
      if (Modernizr.touch) {
        events["touchstart"] = "startDrag";
        events["touchstart .contextual"] = function(e) {e.stopPropagation(); };
      }
      else {
        events["mousedown"] = "startDrag";
        events["mousedown .contextual"] = function(e) {e.stopPropagation(); };
      }
      return events;
    },

    /**
     * {@inheritdoc}
     */
    initialize: function (options) {
      // Add listeners.
      this.listenTo(this.model, 'change:isDragging', this.render);
      this.listenTo(this.model, 'stopDrag', this.stopDrag);
      this.listenTo(this.model, 'destroy', this.remove);

      // Init data.
      this.$ghost = $(Drupal.theme('editUiGhostBlock'));

      // Init model state.
      this.model.setBlock(this.$el);
      this.model.visible();
    },

    /**
     * {@inheritdoc}
     */
    render: function() {
      this.$el.toggleClass(this.hiddenClass, this.model.get('isDragging'));
    },

    /**
     * Start drag.
     */
    startDrag: function(event) {
      var width, height, offset, marginTop, marginBottom;
      if (!Drupal.editUiToolbar.models.toolbarModel.get('isOpen')) {
        return;
      }
      if (event.type === 'mousedown' && event.button !== 0) {
        return;
      }
      event.preventDefault();

      // Get data.
      width = this.$el.outerWidth();
      height = this.$el.outerHeight();
      offset = this.$el.offset();
      marginTop = this.$el.css('marginTop');
      marginBottom = this.$el.css('marginBottom');

      // Trigger startDrag event.
      this.$el.trigger("startDrag", [
        this.model,
        Drupal.editUiUtils.getPosition(event),
        width,
        height,
        offset
      ]);

      // Ghost element.
      this.$ghost.insertAfter(this.$el);
      this.$ghost.css({
        width: width,
        height: height,
        marginTop: marginTop,
        marginBottom: marginBottom
      });
    },

    /**
     * Stop drag.
     */
    stopDrag: function() {
      // Ghost element.
      this.$ghost.detach();
    },

    /**
     * Delete block.
     *
     * @param {Event} event
     *   Event object.
     */
    deleteBlock: function(event) {
      event.preventDefault();
      var message = Drupal.formatString(strings.confirmDelete, {'@name': this.model.get('label')});

      if (confirm(message)) {
        this.model.destroy({success: Drupal.editUiUtils.callAjaxCommands});
      }
    }
  });

  /**
   * Ghost block template.
   *
   * @return String
   *   A string representing a DOM fragment.
   */
  Drupal.theme.editUiGhostBlock = function () {
    return '<div class="edit-ui__ghost-block"></div>';
  };

}(Drupal, Backbone, jQuery, Modernizr));
