/**
 * @file
 * A Backbone view for the edit_ui body element.
 */

(function (Drupal, Backbone, $, Modernizr) {
  "use strict";

  /**
   * Backbone view for the edit_ui body.
   */
  Drupal.editUiBlock.BodyVisualView = Backbone.View.extend({
    /**
     * Main element.
     */
    el: 'body',

    /**
     * Dom elements events.
     */
    events: {
      "startDrag": "startDrag"
    },

    /**
     * {@inheritdoc}
     */
    initialize: function (options) {
      // Add listeners.
      this.listenTo(this.collection, 'add', this.blockAdded);

      // Get data.
      this.$draggingBlock = $('#edit-ui__dragging-block');

      // Initialize collection.
      this.collection.fetch();
    },

    /**
     * {@inheritdoc}
     */
    remove: function() {
      this.$el.off('.editUiBlockBodyVisualView');
      $(document).off('.editUiBlockBodyVisualView');
      Backbone.View.prototype.remove.apply(this, arguments);
    },

    /**
     * Block added callback.
     *
     * @param {Drupal.editUiBlock.BlockModel} block
     *   Block model instance.
     */
    blockAdded: function (block) {
      var id, $el, view;

      if (block.get('plugin_id') === 'system_main_block') {
        return;
      }

      id = block.get('html_id');
      $el = $('#' + id);

      if ($el.length > 0) {
        // Init the block view.
        view = new Drupal.editUiBlock.BlockVisualView({
          el: $el,
          model: block
        });
        Drupal.editUiBlock.views.blockVisualViews.push(view);
      }
    },

    /**
     * Reset dragging block style.
     */
    resetStyle: function() {
      this.$draggingBlock.css({
        width: '',
        height: '',
        marginTop: '',
        marginLeft: '',
        borderRadius: '',
        opacity: ''
      });
    },

    /**
     * Start drag.
     *
     * @param {Event} event
     *   The event object.
     * @param {Drupal.editUiBlock.BlockModel} block
     *   The block model of the dragged element.
     * @param {object} position
     *   The mouse or touch position.
     * @param {number} width
     *   Original element width.
     * @param {number} height
     *   Original element height.
     * @param {object} offset
     *   Original element offset.
     */
    startDrag: function(event, block, position, width, height, offset) {
      // Dragging block styles.
      this.$draggingBlock.css({
        width: width,
        height: height,
        top: position.y - 20,
        left: position.x - 20,
        marginTop: -position.y + offset.top + 20,
        marginLeft: -position.x + offset.left + 20,
        borderRadius: 0,
        opacity: 0
      });
      this.$draggingBlock.show();

      // Start dragging animation.
      setTimeout(this.resetStyle.bind(this), 0);

      // Initialize regions data.
      Drupal.editUiRegion.collections.regionCollection.init();

      // Add listeners.
      if (Modernizr.touch) {
        this.$el.on('touchmove.editUiBlockBodyVisualView', $.proxy(this.drag, this, block));
        this.$el.on('touchend.editUiBlockBodyVisualView', $.proxy(this.drop, this, block));
      }
      else {
        this.$el.on('mousemove.editUiBlockBodyVisualView', $.proxy(this.drag, this, block));
        this.$el.on('mouseup.editUiBlockBodyVisualView', $.proxy(this.drop, this, block));
      }

      // Trigger startDrag event.
      block.trigger('startDrag', block, position);
    },

    /**
     * Drag.
     *
     * @param {Drupal.editUiBlock.BlockModel} block
     *   The dropped block.
     * @param {Event} event
     *   The event object.
     */
    drag: function(block, event) {
      var position;
      event.preventDefault();

      // Dragging block styles.
      position = Drupal.editUiUtils.getPosition(event);
      this.$draggingBlock.css({
        top: position.y - 20,
        left: position.x - 20
      });

      // Propagates drag events to region collection.
      Drupal.editUiRegion.collections.regionCollection.drag(position.x, position.y);
    },

    /**
     * Drop.
     *
     * @param {Drupal.editUiBlock.BlockModel} block
     *   The dropped block.
     * @param {Event} event
     *   The event object.
     */
    drop: function(block, event) {
      // Propagates drop events to region collection.
      var position = Drupal.editUiUtils.getPosition(event);
      Drupal.editUiRegion.collections.regionCollection.drop(block, position.x, position.y);

      // Dragging block styles.
      this.$draggingBlock.hide();

      // Remove listeners.
      this.$el.off('.editUiBlockBodyVisualView');

      // Trigger stopDrag event.
      block.trigger('stopDrag', block, position);
    }
  });

}(Drupal, Backbone, jQuery, Modernizr));
