/**
 * @file
 * A Backbone view for the edit_ui region element.
 */

(function (Drupal, Backbone, $) {
  "use strict";

  /**
   * Backbone view for the edit_ui region.
   */
  Drupal.editUiRegion.RegionVisualView = Backbone.View.extend({
    /**
     * Custom data.
     */
    activeClass: 'active',

    /**
     * {@inheritdoc}
     */
    initialize: function (options) {
      // Add listeners.
      this.listenTo(this.model, 'change:isActive', this.render);
      this.listenTo(this.model, 'init', this.initData);
      this.listenTo(this.model, 'dragOut', this.dragOut);
      this.listenTo(this.model, 'dragIn', this.dragIn);
      this.listenTo(this.model, 'drop', this.drop);
      this.listenTo(this.model, 'addBlock', this.addBlock);

      // Init model state.
      this.model.setBlock(this.$el);
    },

    /**
     * {@inheritdoc}
     */
    render: function() {
      this.$el.toggleClass(this.activeClass, this.model.get('isActive'));
    },

    /**
     * Init region data.
     */
    initData: function() {
      var width = this.$el.width();
      var height = this.$el.height();
      var offset = this.$el.offset();
      this.left = offset.left;
      this.right = offset.left + width;
      this.top = offset.top;
      this.bottom = offset.top + height;
    },

    /**
     * DragOut event callback.
     */
    dragOut: function(region, args) {
      if (args.x > this.left && args.x < this.right && args.y > this.top && args.y < this.bottom) {
        // Enter the region.
        Drupal.editUiRegion.collections.regionCollection.setActiveRegion(region);
      }
    },

    /**
     * DragIn event callback.
     */
    dragIn: function(region, args) {
      if (args.x < this.left || args.x > this.right || args.y < this.top || args.y > this.bottom) {
        // Leave the region.
        Drupal.editUiRegion.collections.regionCollection.setActiveRegion();
      }
      else {
        // TODO: Manage block order.
      }
    },

    /**
     * Drop event callback.
     *
     * @param {Drupal.editUiRegion.RegionModel} region
     *   The block model of the region.
     * @param {Drupal.editUiBlock.BlockModel} block
     *   The block model of the dropped element.
     */
    drop: function(region, block) {
      var weight = 0;
      var blocks = Drupal.editUiBlock.collections.blockCollection.where({region: this.model.get('region')});

      // Fetch the new block weight.
      blocks.forEach(function(block) {
        if (block.get('weight') > weight) {
          weight = block.get('weight');
        }
      });

      // Update block state.
      block.drop(this.model.get('region'), weight);

      // Trigger addBlock event.
      this.model.trigger('addBlock', this.model, block);
    },

    /**
     * Add block event callback.
     *
     * @param {Drupal.editUiRegion.RegionModel} model
     *   The model of the region.
     * @param {Drupal.editUiBlock.BlockModel} addedBlock
     *   The model of the block.
     * @param {string} content
     *   The block content.
     */
    addBlock: function(model, addedBlock, content) {
      var beforeBlock, $block;
      var cid = addedBlock.cid;
      var weight = addedBlock.get('weight');
      var region = addedBlock.get('region');
      var blocks = Drupal.editUiBlock.collections.blockCollection.where({region: region});

      // Find the block just before.
      blocks.forEach(function(block) {
        if (block.get('isVisible') && block.get('weight') <= weight && block.cid !== cid) {
          beforeBlock = block.get('block');
        }
      });

      // If not get the region.
      if (!beforeBlock) {
        beforeBlock = this.$el;
      }

      // Insert block.
      if (content) {
        // In that case it's a new block.
        $block = $(content);
        beforeBlock.after($block);
        Drupal.attachBehaviors($block.get(0));
        Drupal.editUiBlock.collections.blockCollection.add(addedBlock);
      }
      else {
        // In that case just move the existing block.
        beforeBlock.after(addedBlock.get('block'));
      }
    }
  });

}(Drupal, Backbone, jQuery));
