/**
 * @file
 * A Backbone view for the edit_ui toolbar element.
 */

(function (Drupal, Backbone, $) {
  "use strict";

  /**
   * Backbone view for the edit_ui toolbar.
   */
  Drupal.editUiToolbar.ToolbarVisualView = Backbone.View.extend({
    /**
     * Custom data.
     */
    hiddenClass: 'hidden',
    activeClass: 'active',

    /**
     * Dom elements events.
     */
    events: function() {
      var events = {};
      if (Modernizr.touch) {
        events["touchstart .js-edit-ui__add-block"] = "startDrag";
      }
      else {
        events["mousedown .js-edit-ui__add-block"] = "startDrag";
      }
      return events;
    },

    /**
     * {@inheritdoc}
     */
    initialize: function (options) {
      this.toolbarModel = options.toolbarModel;

      // Add listeners.
      this.listenTo(this.model, 'change', this.render);

      // Initialize AJAX links.
      $('.js-edit-ui__add-block', this.$el).each(this.initAjax);
    },

    /**
     * {@inheritdoc}
     */
    render: function () {
      this.$el
        .css({top: this.model.get('topOffset')})
        .toggleClass(this.hiddenClass, !this.model.get('isOpen'));
      return this;
    },

    /**
     * Init link AJAX.
     *
     * @param {number} index
     *   Collection index.
     * @param {Element} link
     *   Collection element.
     */
    initAjax: function (index, link) {
      var $link = $(link);
      var id = $link.attr('id');
      var element_settings = {
        url: $link.attr('href'),
        event: 'click',
        progress: {type: 'throbber'},
        accepts: 'application/vnd.drupal-modal',
        dialog: {
          width: 800
        },
        submit: {
          'js': true,
          'module': 'edit_ui'
        }
      };
      Drupal.ajax[id] = new Drupal.ajax(id, link, element_settings);
    },

    /**
     * Start drag.
     *
     * @param {Event} event
     *   The event object.
     */
    startDrag: function(event) {
      var block;
      if (!this.model.get('isOpen')) {
        return;
      }
      if (event.type === 'mousedown' && event.button !== 0) {
        return;
      }
      event.preventDefault();

      // Create model.
      this.$target = $(event.target);
      block = new Drupal.editUiBlock.BlockModel();
      Drupal.editUiBlock.models.newBlockModel = block;

      // Add listener.
      this.listenTo(block, 'stopDrag', this.stopDrag);

      // Add classes.
      this.$el.addClass(this.activeClass);
      this.$target.addClass(this.activeClass);

      // Trigger startDrag event.
      this.$el.trigger("startDrag", [
        block,
        Drupal.editUiUtils.getPosition(event),
        this.$target.outerWidth(),
        this.$target.outerHeight(),
        this.$target.offset()
      ]);
    },

    /**
     * Stop drag.
     *
     * @param {Drupal.editUiBlock.BlockModel} block
     *   The dropped block.
     */
    stopDrag: function(block) {
      // Remove listener.
      this.stopListening(block);

      // Remove classes.
      this.$el.removeClass(this.activeClass);
      this.$target.removeClass(this.activeClass);

      if (!block.get('region')) {
        // Block dropped outside a region.
        block.destroy();
        Drupal.editUiBlock.models.newBlockModel = null;
      }
      else {
        // Block dropped inside a region.
        Drupal.ajax[this.$target.attr('id')].submit.region = block.get('region');
        Drupal.ajax[this.$target.attr('id')].submit.weight = block.get('weight');
        this.$target.trigger('click');
      }
    }
  });

}(Drupal, Backbone, jQuery));
